/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.client;

import static bufferings.ktr.wjr.shared.util.AssertionUtil.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import bufferings.ktr.wjr.client.service.KtrWjrService;
import bufferings.ktr.wjr.client.service.KtrWjrServiceAsync;
import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStore;
import bufferings.ktr.wjr.shared.model.TestStoreConfig;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * KtrWjrのプレゼンターです。
 * 
 * @author bufferings[at]gmail.com
 */
public class KtrWjrPresenter {

  /** KtrWjrサービス呼び出しのプロキシ */
  protected KtrWjrServiceAsync service = GWT.create(KtrWjrService.class);

  /** ビュー */
  protected KtrWjrView view;

  /** テストストア設定マップ */
  protected Map<String, TestStoreConfig> testsStoreConfigs =
    new HashMap<String, TestStoreConfig>();

  /** テストストアマップ */
  protected Map<String, TestStore> testStores =
    new HashMap<String, TestStore>();

  /** テスト実行中フラグ(実行しているストアの名前を保持します) */
  protected List<String> runningTestFlags = new ArrayList<String>();

  /** テスト実行キャンセルフラグ(キャンセルするストアの名前を保持します) */
  protected List<String> cancelTestFlags = new ArrayList<String>();

  /**
   * ビューを設定してインスタンスを生成します。
   * 
   * @param view
   *          ビュー
   * @throws NullPointerException
   *           ビューがnullの場合
   */
  public KtrWjrPresenter(KtrWjrView view) {
    checkNotNull(view);
    this.view = view;
  }

  public Map<String, TestStoreConfig> getTestStoreConfigs() {
    return testsStoreConfigs;
  }

  public TestStore getTestStore(String storeName) {
    return testStores.get(storeName);
  }

  public void loadTestStoreConfigs() {
    checkArgument(testsStoreConfigs.size() == 0);
    service.loadTestStoreConfigs(new AsyncCallback<List<TestStoreConfig>>() {
      public void onFailure(Throwable caught) {
        GWT.log("loadTestStoreConfigs onFailure", caught);
        view.onLoadTestStoreConfigsFailure();
      }

      public void onSuccess(List<TestStoreConfig> result) {
        GWT.log("loadTestStoreConfigs onSuccess", null);

        testsStoreConfigs = new LinkedHashMap<String, TestStoreConfig>();
        for (TestStoreConfig config : result) {
          testsStoreConfigs.put(config.getStoreKey(), config);
        }

        view.onLoadTestStoreConfigsSuccess();
      }
    });
  }

  public void loadTestStore(final String storeKey) {
    checkArgument(!testStores.containsKey(storeKey));
    service.loadTestStore(storeKey, new AsyncCallback<TestStore>() {
      public void onFailure(Throwable caught) {
        GWT.log("loadTestStore onFailure", caught);
        view.onLoadTestStoreFailure(storeKey);
      }

      public void onSuccess(TestStore result) {
        GWT.log("loadTestStore onSuccess", null);
        result.getRootSuite().updateSummeryAndState(result);
        testStores.put(storeKey, result);
        view.onLoadTestStoreSuccess(storeKey);
      }
    });
  }

  public void runTests(String storeName, List<TestCaseModel> testCases) {
    checkArgument(!runningTestFlags.contains(storeName));
    try {
      runningTestFlags.add(storeName);
      cancelTestFlags.remove(storeName);
      runTestsInner(storeName, testCases, 0, testsStoreConfigs
        .get(storeName)
        .getMaxTestCaseCountPerRequest());
    } catch (RuntimeException e) {
      runningTestFlags.remove(storeName);
      throw e;
    }
  }

  public void cancelTests(String storeName) {
    checkArgument(runningTestFlags.contains(storeName));
    cancelTestFlags.add(storeName);
  }

  protected void runTestsInner(final String storeName,
      final List<TestCaseModel> testCases, final int fromIndex,
      final int countPerRequest) {
    checkArgument(countPerRequest > 0);
    final int toIndex = Math.min(fromIndex + countPerRequest, testCases.size());

    List<TestCaseModel> subList = new ArrayList<TestCaseModel>();
    for (int i = fromIndex; i < toIndex; i++) {
      subList.add(testCases.get(i));
    }

    service.runTests(subList, new AsyncCallback<List<TestCaseModel>>() {

      public void onFailure(Throwable caught) {
        GWT.log("runTestsInner onFailure", caught);
        cancelTestFlags.remove(storeName);
        runningTestFlags.remove(storeName);
        view.onRunTestsFailure(storeName);
      }

      public void onSuccess(List<TestCaseModel> result) {
        GWT.log("runTestsInner onSuccess", null);
        testStores.get(storeName).set(result);

        if (cancelTestFlags.contains(storeName)) {
          // キャンセル
          TestStore store = testStores.get(storeName);
          store.getRootSuite().updateSummeryAndState(store);
          cancelTestFlags.remove(storeName);
          runningTestFlags.remove(storeName);
          view.onRunTestsCancel(storeName);
        } else {
          if (toIndex == testCases.size()) {
            // 終了
            TestStore store = testStores.get(storeName);
            store.getRootSuite().updateSummeryAndState(store);
            cancelTestFlags.remove(storeName);
            runningTestFlags.remove(storeName);
            view.onRunTestsSuccess(storeName);
          } else {
            TestStore store = testStores.get(storeName);
            store.getRootSuite().updateSummeryAndState(store);
            view.onRunTestsUpdate(storeName);

            runTestsInner(storeName, testCases, toIndex, countPerRequest);
          }
        }
      }
    });
  }

}
