/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
 package bufferings.ktr.wjr.server.service;

import static bufferings.ktr.wjr.shared.util.AssertionUtil.*;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.mortbay.log.Log;

import bufferings.ktr.wjr.shared.model.TestStoreConfig;

public class TestStoreConfigLoader {

  protected static final String CONFIG_RESOURCE_FILE = "ktrwjr.properties";

  protected static final String CONFIG_ITEM_STORENAME = "storeName";

  protected static final String CONFIG_ITEM_MAXCOUNT =
    "maxTestCaseCountPerRequest";

  protected static final String CONFIG_ITEM_CLASSESDIR = "classesDirectory";

  protected static final String CONFIG_ITEM_ROOTPACKAGE = "rootPackage";

  protected static final String CONFIG_ITEM_DESCRIPTION = "description";

  public List<TestStoreConfig> loadTestStoreConfigs() {
    Properties props = loadProperties();
    Enumeration<Object> propKeys = props.keys();

    Map<String, TestStoreConfig> configMap =
      new LinkedHashMap<String, TestStoreConfig>();
    while (propKeys.hasMoreElements()) {
      String propKey = (String) propKeys.nextElement();

      ConfigKeyInfo keyInfo = new ConfigKeyInfo(propKey);
      String storeKey = keyInfo.getStoreKey();

      TestStoreConfig config;
      if (configMap.containsKey(storeKey)) {
        config = configMap.get(storeKey);
      } else {
        config = new TestStoreConfig(storeKey);
        configMap.put(storeKey, config);
      }

      setConfigAttribute(config, keyInfo, props.getProperty(propKey));
    }
    return new ArrayList<TestStoreConfig>(configMap.values());
  }

  public TestStoreConfig loadTestStoreConfig(String storeKey) {
    checkNotNull(storeKey);

    Properties props = loadProperties();
    Enumeration<Object> propKeys = props.keys();

    TestStoreConfig config = null;
    while (propKeys.hasMoreElements()) {
      String propKey = (String) propKeys.nextElement();
      ConfigKeyInfo keyInfo = new ConfigKeyInfo(propKey);
      if (!storeKey.equals(keyInfo.getStoreKey())) {
        continue;
      }

      if (config == null) {
        config = new TestStoreConfig(storeKey);
      }

      setConfigAttribute(config, keyInfo, props.getProperty(propKey));
    }
    return config;
  }

  protected void setConfigAttribute(TestStoreConfig config,
      ConfigKeyInfo keyInfo, String value) {
    String configItemName = keyInfo.getItemName();
    if (CONFIG_ITEM_STORENAME.equals(configItemName)) {
      config.setStoreName(value);
    } else if (CONFIG_ITEM_MAXCOUNT.equals(configItemName)) {
      int maxCnt;
      try {
        maxCnt = Integer.parseInt(value);
        config.setMaxTestCaseCountPerRequest(maxCnt);
      } catch (NumberFormatException e) {
        Log.warn("Invalid number format.(" + value + ")");
      }
    } else if (CONFIG_ITEM_CLASSESDIR.equals(configItemName)) {
      config.setClassesDirectory(value);
    } else if (CONFIG_ITEM_ROOTPACKAGE.equals(configItemName)) {
      config.setRootPackage(value);
    } else if (CONFIG_ITEM_DESCRIPTION.equals(configItemName)) {
      config.setDescription(value);
    } else {
      Log.warn("Unknown item name.(" + configItemName + ")");
    }
  }

  protected Properties loadProperties() {
    Properties properties = new Properties();
    InputStream is = null;
    try {
      ClassLoader cl = Thread.currentThread().getContextClassLoader();
      is = cl.getResourceAsStream(CONFIG_RESOURCE_FILE);
      if (is != null) {
        try {
          properties.load(is);
        } catch (IOException e) {
          Log.warn("Fails to load configuration resource file .("
            + CONFIG_RESOURCE_FILE
            + ")", e);
          e.printStackTrace();
        }
      } else {
        Log.warn("Configuration resource file not found.("
          + CONFIG_RESOURCE_FILE
          + ")");
      }
    } finally {
      if (is != null) {
        try {
          is.close();
        } catch (Exception e) {
          // ignore
        }
      }
    }
    return properties;
  }

  protected class ConfigKeyInfo {
    protected static final String SEPARATOR = ".";
    protected static final String SEPARATOR_REGEX = "\\.";

    protected String storeKey;
    protected String itemName;

    public ConfigKeyInfo(String propKey) {
      checkNotNull(propKey);
      checkArgument(propKey.length() > 0);
      checkArgument(propKey.contains(SEPARATOR));

      String[] splits = propKey.split(SEPARATOR_REGEX);
      checkArgument(splits.length == 2);

      storeKey = splits[0];
      itemName = splits[1];
    }

    public String getStoreKey() {
      return storeKey;
    }

    public String getItemName() {
      return itemName;
    }
  }
}
