/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.server.service;

import static bufferings.ktr.wjr.shared.util.AssertionUtil.*;

import java.io.File;
import java.io.FileFilter;
import java.util.logging.Level;
import java.util.logging.Logger;

import bufferings.ktr.wjr.server.service.junit3.JUnit3Checker;
import bufferings.ktr.wjr.server.service.junit3.TestCaseChecker3;
import bufferings.ktr.wjr.server.service.junit4.JUnit4Checker;
import bufferings.ktr.wjr.server.service.junit4.TestCaseChecker4;
import bufferings.ktr.wjr.shared.model.TestStore;

public class TestStoreLoader {
  private static final Logger logger =
    Logger.getLogger(TestStoreLoader.class.getName());

  protected static final String CLASSFILE_EXT = ".class";

  protected static final String INNER_CLASS_SEPARATOR = "$";

  protected static final String FILE_SEPARATOR = "/";

  protected static final String FILE_SEPARATOR_REGEX = "[\\\\/]";

  protected static final String PACKAGE_SEPARATOR = ".";

  protected static final String PACKAGE_SEPARATOR_REGEX = "\\.";

  protected interface FindTestsVisitor {
    public void visit(File file);
  }

  protected class FindTestsVisitorImpl implements FindTestsVisitor {

    protected TestStore store;

    protected ClassLoader cl;

    protected String classesDirectory;

    public FindTestsVisitorImpl(TestStore store, String classesDirectory,
        ClassLoader cl) {
      checkNotNull(store);
      checkNotNull(cl);
      checkNotNull(classesDirectory);
      this.store = store;
      this.cl = cl;
      this.classesDirectory = classesDirectory;
    }

    public void visit(File file) {
      String name = file.getPath();
      name =
        name.substring(classesDirectory.length() + 1, name.length()
          - CLASSFILE_EXT.length());
      name = name.replaceAll(FILE_SEPARATOR_REGEX, PACKAGE_SEPARATOR);
      logger.fine("visit(name=" + name + ")");

      Class<?> c;
      try {
        c = cl.loadClass(name);
        boolean isAdded = false;
        if (JUnit4Checker.isAvailable()) {
          isAdded = new TestCaseChecker4().checkAndAddTestCase(store, c);
        }
        if (!isAdded && JUnit3Checker.isAvailable()) {
          new TestCaseChecker3().checkAndAddTestCase(store, c);
        }
      } catch (NoClassDefFoundError e) {
        logger.log(Level.WARNING, "Class not found(" + name + ")", e);
      } catch (ClassNotFoundException e) {
        logger.log(Level.WARNING, "Class not found(" + name + ")", e);
      }
    }
  }

  protected class ClassFileFilter implements FileFilter {

    public boolean accept(File pathname) {
      return !pathname.isDirectory()
        && pathname.getName().endsWith(CLASSFILE_EXT)
        && !pathname.getName().contains(INNER_CLASS_SEPARATOR);
    }

  }

  protected class DirectoryFileFilter implements FileFilter {

    public boolean accept(File pathname) {
      return pathname.isDirectory();
    }

  }

  public TestStore loadTestStore(String storeKey, String storeName,
      String classesDirectory, String rootPackage) {

    TestStore store = new TestStore(storeKey);
    ClassLoader cl = Thread.currentThread().getContextClassLoader();

    String rootPackageDir =
      rootPackage.replaceAll(PACKAGE_SEPARATOR_REGEX, FILE_SEPARATOR);
    File searchDir =
      new File(classesDirectory + FILE_SEPARATOR + rootPackageDir);
    FindTestsVisitor visitor =
      new FindTestsVisitorImpl(store, classesDirectory, cl);
    findTests(searchDir, visitor);

    return store;
  }

  protected void findTests(File dir, FindTestsVisitor visitor) {
    checkArgument(dir.exists());
    checkArgument(dir.isDirectory());

    File[] files = dir.listFiles(new ClassFileFilter());
    for (File f : files) {
      visitor.visit(f);
    }

    files = dir.listFiles(new DirectoryFileFilter());
    for (File f : files) {
      findTests(f, visitor);
    }
  }

}
