/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.shared.model;

import static bufferings.ktr.wjr.shared.util.AssertionUtil.*;

/**
 * A test case model.
 * <p>
 * A parent‐child relationship is not placed in this class. It is in the
 * {@link TestStore}.
 * </p>
 * 
 * @author bufferings[at]gmail.com
 */
public class TestCaseModel implements TestStoreItem {

  private static final long serialVersionUID = 1L;

  /**
   * a separator combining class name and method name for key
   */
  protected static final String CLASS_METHOD_SEPARATOR = "#";

  /** a test class name */
  protected String className;

  /** a test method name */
  protected String methodName;

  /** a test run state */
  protected State state = State.NOT_YET;

  /** a test result stack trace string */
  protected String trace = null;

  /** a time running the test took[ms] */
  protected String time = null;

  /**
   * Default constructor for GWT-RPC serialization.
   * <p>
   * This is not used usually.
   * </p>
   */
  @Deprecated
  public TestCaseModel() {
  }

  /**
   * Constructor
   * 
   * @param className
   *          test case class name
   * @param methodName
   *          test case method name
   * @throws NullPointerException
   *           when the class name or the method name is null
   * @throws IllegalArgumentException
   *           when the class name or the method name is empty string
   */
  public TestCaseModel(String className, String methodName) {
    checkNotNull(className);
    checkNotNull(methodName);
    checkArgument(className.length() > 0);
    checkArgument(methodName.length() > 0);

    this.className = className;
    this.methodName = methodName;
  }

  /** {@inheritDoc} */
  public String getKey() {
    return className + CLASS_METHOD_SEPARATOR + methodName;
  }

  /** {@inheritDoc} */
  public String getName() {
    return methodName;
  }

  /**
   * Gets the test run state.
   * 
   * @return the test run state
   */
  public State getState() {
    return state;
  }

  /**
   * Gets the test class name.
   * 
   * @return the test class name
   */
  public String getClassName() {
    return className;
  }

  /**
   * Gets the test method name.
   * 
   * @return the test method name
   */
  public String getMethodName() {
    return methodName;
  }

  /**
   * Sets the test run state.
   * 
   * @param state
   *          the test run state
   */
  public void setState(State state) {
    this.state = state;
  }

  /**
   * Gets the test result stack trace string.
   * 
   * @return the test result stack trace string
   */
  public String getTrace() {
    return trace;
  }

  /**
   * Sets the test result stack trace string
   * 
   * @param trace
   *          the test result stack trace string
   */
  public void setTrace(String trace) {
    this.trace = trace;
  }

  /**
   * Gets the time running the test took.
   * 
   * @return the time running the test took
   */
  public String getTime() {
    return time;
  }

  /**
   * Sets the time running the test took.
   * 
   * @param time
   *          the time running the test took
   */
  public void setTime(String time) {
    this.time = time;
  }

  public void clearResult() {
    state = State.NOT_YET;
    trace = null;
    time = null;
  }

  /** {@inheritDoc} */
  @Override
  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append("TestCaseModel(");
    sb.append("className=").append(className).append(", ");
    sb.append("methodName=").append(methodName).append(", ");
    sb.append("state=").append(state).append("),");
    sb.append("trace=").append(trace).append(")");
    sb.append("time=").append(time).append(")");
    return sb.toString();
  }

}
