/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.shared.model;

import static bufferings.ktr.wjr.shared.util.AssertionUtil.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * A store which keeps test store items and their relationships.
 * 
 * @author bufferings[at]gmail.com
 */
public class TestStore implements Serializable {

  private static final long serialVersionUID = 1L;

  /** a key of the root suite */
  protected static final String ROOT_TESTSUITE_KEY = "";

  /** a name of the root suite */
  protected static final String ROOT_TESTSUITE_NAME = "All Tests";

  /** a key of this test store */
  protected String storeKey;

  /** a map which keeps test store items */
  protected Map<String, TestStoreItem> itemStore;

  /** a map which keeps the filtered results */
  protected Map<String, Boolean> filterResultStore;

  /** a map which keeps the parent-child relationship */
  protected Map<String, String> parentStore;

  /** root test suite */
  protected TestSuiteModel rootTestSuite;

  /** test case model filter */
  protected TestCaseModelFilter filter;

  /**
   * Default constructor for GWT-RPC serialization.
   * <p>
   * This is not used usually.
   * </p>
   */
  @Deprecated
  public TestStore() {
  }

  /**
   * Constructor
   * 
   * @param storeKey
   *          store key
   * @thwows NullPointerException when the store key is null
   */
  public TestStore(String storeKey) {
    checkNotNull(storeKey);

    this.storeKey = storeKey;

    itemStore = new HashMap<String, TestStoreItem>();
    filterResultStore = new HashMap<String, Boolean>();
    parentStore = new HashMap<String, String>();

    filter = new AllPassFilter();
    rootTestSuite = new TestSuiteModel(ROOT_TESTSUITE_KEY, ROOT_TESTSUITE_NAME);
    itemStore.put(rootTestSuite.getKey(), rootTestSuite);
  }

  /**
   * Gets the store key.
   * 
   * @return the store key
   */
  public String getStoreKey() {
    return storeKey;
  }

  /**
   * Sets the filter, and applys the filter.
   * <p>
   * If the filter is null, {@link AllPassFilter} is set.
   * </p>
   * 
   * @param filter
   *          the filter
   */
  public void setFilter(TestCaseModelFilter filter) {
    if (filter == null) {
      this.filter = new AllPassFilter();
    } else {
      this.filter = filter;
    }
    applyFilter();
  }

  /**
   * Applys the filter to test store items.
   */
  public void applyFilter() {
    for (TestStoreItem item : itemStore.values()) {
      applyFilter(item);
    }
  }

  /**
   * Applys the filter to the test item.
   * <p>
   * The filter doesn't affect test suites.
   * </p>
   * 
   * @param item
   *          the test store item to apply filter
   * @throws NullPointerException
   *           when the test item is null or not in this store
   */
  protected void applyFilter(TestStoreItem item) {
    checkNotNull(item);
    checkNotNull(itemStore.get(item.getKey()));

    if (item instanceof TestSuiteModel) {
      filterResultStore.put(item.getKey(), Boolean.TRUE);
    } else {
      TestCaseModel testCase = (TestCaseModel) item;
      filterResultStore.put(testCase.getKey(), filter.match(testCase));
    }
  }

  /**
   * Gets the root test suite.
   * 
   * @return the root test suite
   */
  public TestSuiteModel getRootSuite() {
    return rootTestSuite;
  }

  /**
   * Gets the test store item for the key.
   * <p>
   * If no item found, returns null.
   * </p>
   * 
   * @param key
   *          a key of test store item
   * @return the found test store item
   * @throws NullPointerException
   *           when the key is null
   */
  public TestStoreItem getTestStoreItem(String key) {
    checkNotNull(key);
    return itemStore.get(key);
  }

  /**
   * Gets children list of the specified test suite.
   * <ul>
   * <li>children are filered</li>
   * <li>getting direct children only, not grand children</li>
   * <li>children are sorted by the key string ascending</li>
   * </ul>
   * 
   * @param suiteKey
   *          a key of the test suite where you want to get the children.
   * @return the children list of the specified test suite(filtered and sorted)
   * @throws NullPointerException
   *           when the key is null or not in this store
   * @throws IllegalArgumentException
   *           when the test item for the key is not a type of test suite
   */
  public List<TestStoreItem> getSuiteChildren(String suiteKey) {
    return getSuiteChildren(suiteKey, true);
  }

  /**
   * Gets children list of the specified test suite.
   * <ul>
   * <li>children are filered</li>
   * <li>getting direct children only, not grand children</li>
   * <li>children are sorted by the key string ascending</li>
   * </ul>
   * 
   * @param suiteKey
   *          a key of the test suite where you want to get the children.
   * @param useFilter
   *          true: use filter, false: not use filter(return all children)
   * @return the children list of the specified test suite(filtered and sorted)
   * @throws NullPointerException
   *           when the key is null or not in this store
   * @throws IllegalArgumentException
   *           when the test item for the key is not a type of test suite
   */
  public List<TestStoreItem> getSuiteChildren(String suiteKey, boolean useFilter) {
    checkNotNull(suiteKey);
    TestStoreItem suiteItem = itemStore.get(suiteKey);
    checkNotNull(suiteItem);
    checkArgument(suiteItem instanceof TestSuiteModel);

    List<TestStoreItem> items = new ArrayList<TestStoreItem>();
    for (Entry<String, String> entry : parentStore.entrySet()) {
      if (suiteKey.equals(entry.getValue())) {
        if (!useFilter || filterResultStore.get(entry.getKey())) {
          items.add(itemStore.get(entry.getKey()));
        }
      }
    }

    Collections.sort(items, new Comparator<TestStoreItem>() {

      public int compare(TestStoreItem o1, TestStoreItem o2) {
        return o1.getKey().compareTo(o2.getKey());
      }
    });

    return items;
  }

  /**
   * Adds a test item specifying a parent test suite.
   * 
   * @param suiteKey
   *          a parent test suite key
   * @param item
   *          a test item to add
   * @throws NullPointerException
   *           when the test suite key is null or not in the store
   * @throws IllegalArgumentException
   *           when the test item for the suiteKey is not a type of test suite
   * @throws NullPointerException
   *           when the test item is null
   * @throws NullPointerException
   *           when the key of the test case is null
   * @throws IllegalArgumentException
   *           when the test item key has been already in this store
   */
  public void add(String suiteKey, TestStoreItem item) {
    checkNotNull(suiteKey);
    TestStoreItem suiteItem = itemStore.get(suiteKey);
    checkNotNull(suiteItem);
    checkArgument(suiteItem instanceof TestSuiteModel);
    checkNotNull(item);
    String key = item.getKey();
    checkNotNull(key);
    checkArgument(itemStore.get(key) == null);

    itemStore.put(key, item);
    parentStore.put(key, suiteKey);
    applyFilter(item);
  }

  /**
   * Adds a test suite item as a child of the root suite.
   * 
   * @param item
   *          a test item to add
   * @throws NullPointerException
   *           when the test item is null
   * @throws NullPointerException
   *           when the key of the test case is null
   * @throws IllegalArgumentException
   *           when the test item key has been already in this store
   */
  public void add(TestStoreItem item) {
    add(ROOT_TESTSUITE_KEY, item);
  }

  /**
   * Updates the test case.
   * 
   * @param testCase
   *          a test case to update
   * @throws NullPointerException
   *           when the test case is null
   * @throws NullPointerException
   *           when the key of the test case is null or not in this store
   * @throws IllegalArgumentException
   *           when the test item for the test case key is not type of test case
   */
  public void set(TestCaseModel testCase) {
    checkNotNull(testCase);
    String key = testCase.getKey();
    checkNotNull(key);
    TestStoreItem storeItem = itemStore.get(key);
    checkArgument(storeItem instanceof TestCaseModel);

    itemStore.put(key, testCase);
    applyFilter(testCase);
  }

  /**
   * Updates the test cases.
   * 
   * @param testCase
   *          a test case to update
   * @throws NullPointerException
   *           when the list of test cases is null
   * @see TestStore#update(TestCaseModel)
   */
  public void set(List<TestCaseModel> testCases) {
    checkNotNull(testCases);
    for (TestCaseModel testCase : testCases) {
      set(testCase);
    }
  }

  public void clearResults() {
    for (TestStoreItem item : itemStore.values()) {
      item.clearResult();
    }
  }

}
