/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.shared.model;

import static bufferings.ktr.wjr.shared.util.AssertionUtil.*;

import java.util.List;

/**
 * A test suite model.
 * <p>
 * A parent‐child relationship is not placed in this class. It is in the
 * {@link TestStore}.
 * </p>
 * 
 * @author bufferings[at]gmail.com
 */
public class TestSuiteModel implements TestStoreItem {

  private static final long serialVersionUID = 1L;

  /** suite class name */
  protected String className;

  /** suite display name */
  protected String suiteName;

  /** suite run state */
  protected State state = State.NOT_YET;

  /** total count of test cases */
  protected int totalCount = 0;

  /** succeeded count of test cases */
  protected int successCount = 0;

  /** failed count of test cases */
  protected int failureCount = 0;

  /** error count of test cases */
  protected int errorCount = 0;

  /** not yet run count of test cases */
  protected int notYetCount = 0;

  /**
   * Default constructor for GWT-RPC serialization.
   * <p>
   * This is not used usually.
   * </p>
   */
  @Deprecated
  public TestSuiteModel() {
  }

  /**
   * Constructor
   * 
   * @param className
   *          a suite class name
   * @param suiteName
   *          a suite display name
   */
  public TestSuiteModel(String className, String suiteName) {
    checkNotNull(className);
    checkNotNull(suiteName);
    this.className = className;
    this.suiteName = suiteName;
  }

  /** {@inheritDoc} */

  public String getKey() {
    return className;
  }

  /** {@inheritDoc} */

  public String getName() {
    return suiteName;
  }

  /** {@inheritDoc} */

  public State getState() {
    return state;
  }

  /**
   * Gets the suite class name.
   * 
   * @return the suite class name
   */
  public String getClassName() {
    return className;
  }

  /**
   * Gets the suite display name
   * 
   * @return the suite display name
   */
  public String getSuiteName() {
    return suiteName;
  }

  /**
   * Gets the total count of test cases
   * 
   * @return the total count of test cases
   */
  public int getTotalCount() {
    return totalCount;
  }

  /**
   * Gets the succeeded count of test cases
   * 
   * @return the succeeded count of test cases
   */
  public int getSuccessCount() {
    return successCount;
  }

  /**
   * Gets the failed count of test cases
   * 
   * @return the failed count of test cases
   */
  public int getFailureCount() {
    return failureCount;
  }

  /**
   * Gets the error count of test cases
   * 
   * @return the error count of test cases
   */
  public int getErrorCount() {
    return errorCount;
  }

  /**
   * Gets the not yet run count of test cases
   * 
   * @return not yet run count of test cases
   */
  public int getNotYetCount() {
    return notYetCount;
  }

  /**
   * Updates the test summary and the suite state (recursively)
   * 
   * @param testStore
   *          the test store
   * @throws NullPointerException
   *           when the test store is null
   */
  public void updateSummeryAndState(TestStore testStore) {
    checkNotNull(testStore);

    totalCount = 0;
    successCount = 0;
    failureCount = 0;
    errorCount = 0;
    notYetCount = 0;

    List<TestStoreItem> items =
      testStore.getSuiteChildren(this.getKey(), false);
    for (TestStoreItem item : items) {
      if (item instanceof TestSuiteModel) {
        TestSuiteModel testSuite = (TestSuiteModel) item;
        testSuite.updateSummeryAndState(testStore);

        totalCount += testSuite.getTotalCount();
        successCount += testSuite.getSuccessCount();
        failureCount += testSuite.getFailureCount();
        errorCount += testSuite.getErrorCount();
        notYetCount += testSuite.getNotYetCount();
      } else {
        switch (item.getState()) {
        case NOT_YET:
          notYetCount++;
          break;
        case SUCCESS:
          successCount++;
          break;
        case FAILURE:
          failureCount++;
          break;
        case ERROR:
          errorCount++;
          break;
        default:
          throw new AssertionError("Unknown state.");
        }
        totalCount++;
      }
    }

    if (errorCount > 0) {
      state = State.ERROR;
    } else if (failureCount > 0) {
      state = State.FAILURE;
    } else if (notYetCount > 0) {
      state = State.SUITE_HAS_NOT_YET;
    } else {
      state = State.SUCCESS;
    }
  }

  /** {@inheritDoc} */
  @Override
  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append("TestSuiteModel(");
    sb.append("className=").append(className).append(", ");
    sb.append("suiteName=").append(suiteName).append(", ");
    sb.append("state=").append(state).append(", ");
    sb.append("totalCount=").append(totalCount).append(", ");
    sb.append("successCount=").append(successCount).append(", ");
    sb.append("failureCount=").append(failureCount).append(", ");
    sb.append("errorCount=").append(errorCount).append(", ");
    sb.append("notYetCount=").append(notYetCount).append(")");
    return sb.toString();
  }

  public void clearResult() {
    state = State.NOT_YET;
    successCount = 0;
    failureCount = 0;
    errorCount = 0;
    notYetCount = totalCount;
  }
}
